import pickle

from .logging_utility import logging_debug_decorator
from .logging_utility import logging_info_decorator

@logging_debug_decorator
@logging_info_decorator
def save(game_map, position, filename="savegame.json"):
    """
    Saves game using json dump

    Args:
        game_map ([[str...]...]): game map generated by game_generator module.
        position ([int,int]): player's position
        filename (str): optional filename
    """
    game_map.append(position)
    with open(filename, "wb") as fout:
        pickle.dump(game_map, fout, pickle.HIGHEST_PROTOCOL)
    game_map.pop()


@logging_debug_decorator
@logging_info_decorator
def load(filename="savegame.json"):
    """
    Loads game using json.load

    Args:
        filename (str): optional filename
    Returns:
        [[str...]...] - game_map, [int, int] - position
    """
    try:
        with open(filename, "rb") as fin:
            game_map = pickle.load(fin)
            position = game_map.pop()
            return game_map, position
    except FileNotFoundError:
        print("No savegame file was found!")
        raise
